<?php
/**
 * @category     Addons
 * @package      Admin tool: Pagescode-Check
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Org. e.V.
 * @author       Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Project
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.0.2
 * @lastmodified 2024/10/06
 * @since        File available since 2024/10/06
 * @description  info for wbpagecode
 *
 * function.lib
*/
namespace addon\wbpagecode\Lib;

use bin\{WbAdaptor, SecureTokens};
use bin\helpers\{PreCheck};

#[AllowDynamicProperties]

    class Pagecodecheck {

    /** @var object instance of the application object */
        protected $_oApp;
    /** @var object instance of the database object */
        protected $_oDb ;
    /** @var object instance holds several values from the application global scope */
        protected $_oReg;
    /** @var object instance holds all of the translations */
        protected $_oTrans;


    // -----------------------------------------------------------------------------
    /** prevent class from public instancing and get an object to hold extensions */
    public function __construct()
    {
        // import global vars and objects
        $this->_oReg   = WbAdaptor::getInstance();
        $this->_oDb    = $this->_oReg->getDatabase();
        $this->_oTrans = $this->_oReg->getTranslate();
        $this->_oApp   = $this->_oReg->getApplication();
    }
    // -----------------------------------------------------------------------------
    public function getSettings($iSectionId) {
        return $this->_readSettings($iSectionId);
    }
    // -----------------------------------------------------------------------------

###############################################################################
// read settings for this section (_readSettings)
###############################################################################
    /** read default setting for this section */
         protected function _readSettings($iSectionId){
            global $aDefaults, $oAddonReg, $section_id, $page_id, $sAddonPath ;
//                    $oDb = \database::getInstance();
                $aPageCodeSettings  = [];
                $sql  = 'SELECT `s_name`, `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_pagecode_settings` '
                      . 'WHERE `section_id` = ' . $iSectionId;
//                        echo $sql;
                if ($query = $this->_oDb->query($sql)){
                    while ($row = $query->fetchAssoc()) {
                        $aPageCodeSettings[$row['s_name']] = $row['s_value'];
                    }
                }

                // insert default_settings if no settings for this section avaiable
                if (sizeof($aPageCodeSettings) == 0){
                    $aFieldsList = array(
                        'section_id' => 0,
                        'page_id'    => 0,
                    );
                    /* add default setting if not set for this section */
                    if (\is_array($aDefaults)) {
                        $aTmp = \array_diff_key($aDefaults, $aPageCodeSettings);
                        if ($aTmp){
                            UpdateKeyValue('mod_pagecode_settings', $aDefaults, '', $aFieldsList);
                        }
                    }
                }
//                echo "<pre>";
//                print_r($aPageCodeSettings);
//                echo "</pre>";    
        return $aPageCodeSettings;
        }
    // end of static public function ('_readSettings')


################################################################################
// count all Pagess (CountAllPages)
###############################################################################
        static public function CountAllPages(){
            global $page_id;
            $oDb = \database::getInstance();
            $sql= 'SELECT COUNT(*) FROM `'.TABLE_PREFIX.'pages` ';
            $iCountAllPages = $oDb->get_one($sql);
            return (@$oDb->get_error()?:$iCountAllPages);
        }

    // end of static public function ('CountAllPages')
################################################################################
// count all Pagess (getPageCodeLanguage)
###############################################################################
        static public function getPageCodeLanguage($iPageCodeId){
            global $page_id;
            $oDb = \database::getInstance();
            $sql= 'SELECT `language` FROM `'.TABLE_PREFIX.'pages` '
                    .'WHERE `page_id` = '.(int)$iPageCodeId;
            $iPageCodeLanguage = $oDb->get_one($sql);
            return (@$oDb->get_error()?:$iPageCodeLanguage);
        }
        // end of static public function ('getPageCodeLanguage')   
################################################################################
// count all Pagess (CountAllPages)
###############################################################################
        static public function IsPublished($iPageId){
            global $page_id;
            $aResult=[];
            $oDb = \database::getInstance();
            $sql= 'SELECT  `module`, MAX(`publ_start` + `publ_end`) AS `published` '
                . 'FROM `'.TABLE_PREFIX.'sections` '
                . 'WHERE `page_id` = ' . (int)$iPageId.' '
                . 'GROUP BY `module` ';
    //        echo $sql;
            if ($query = $oDb->query($sql)){
                while ($row = $query->fetchAssoc()) {
                    $aResult['published'] = $row['published'];
                    $aResult['module']    = $row['module'];
                }
            }
            return (@$oDb->get_error()?:$aResult);
        }

################################################################################
// count used languages (CountLanguages)
###############################################################################
        static public function CountLanguages(){
            global $page_id;
            $aResult=[];
            $oDb = \database::getInstance();    

            $default_language = DEFAULT_LANGUAGE;
            $sqlPcSet = 'SELECT COUNT(DISTINCT `language`)  '
                    .   'FROM `'. TABLE_PREFIX .'pages` '
                    .   'WHERE `visibility` NOT IN (\'none\', \'deleted\') '
    //                . 'GROUP BY   `language` '
                    . '';
    //        echo "<br>".$sqlPcSet;
            $iCountLanguages = $oDb->get_one($sqlPcSet);
            return (@$oDb->get_error()?:$iCountLanguages);

        } // end CountLanguages  

    
################################################################################
// count all Pagess (getPageTreeList)
###############################################################################
        static public function getPageTreeList(int $parent, $tree){
            $oReg = WbAdaptor::getInstance();
            $oDb = $oReg->getDataBase();
            global $oDb, $admin, $oTrans;    

            $aResult = [];
            $refs= [];
            $aLangPages = $aPages =  null;
            $default_language = DEFAULT_LANGUAGE;
            $sqlSet = '
            SELECT
            `page_id`,`parent`,`root_parent`,`level`,`link`,`page_trail`,`position`
            ,`language`,`visibility`,`page_title`,`menu_title`,`page_code`,`admin_groups`,`admin_users`
            FROM `'.$oReg->TablePrefix.'pages`
            WHERE `parent`='.(int)$parent.'

            ORDER BY  `position`
            ';   
    //      echo $sql."<br>";

                if ($query_page = $oDb->query($sqlSet)) {
                    while(!\is_null($page = $query_page->fetchRow(\MYSQLI_ASSOC))) {

                        $sPageCodeLanguage = Pagecodecheck::getPageCodeLanguage($page['page_code']);

                        $tree[$page['page_id']]['page_id']             = $page['page_id'];
                        $tree[$page['page_id']]['parent']              = $page['parent'];
                        $tree[$page['page_id']]['root_parent']         = $page['root_parent'];
                        $tree[$page['page_id']]['level']               = $page['level'];
                        $tree[$page['page_id']]['link']                = $page['link'];  //
                        $tree[$page['page_id']]['page_trail']          = $page['page_trail'];  //
                        $tree[$page['page_id']]['position']            = $page['position'];  //
                        $tree[$page['page_id']]['language']            = $page['language'];
                        $tree[$page['page_id']]['visibility']          = $page['visibility'];
                        $tree[$page['page_id']]['page_title']          = $page['page_title'];
                        $tree[$page['page_id']]['menu_title']          = $page['menu_title'];
                        $tree[$page['page_id']]['page_code']           = $page['page_code'];
                        $tree[$page['page_id']]['admin_groups']        = $page['admin_groups'];
                        $tree[$page['page_id']]['admin_users']         = $page['admin_users'];
                        $tree[$page['page_id']]['indent']              = $page['level']*30;
                        $tree[$page['page_id']]['frontend_link']       = PAGES_DIRECTORY . $page['link'] . PAGE_EXTENSION;
                        $tree[$page['page_id']]['modify_page_url']     = '../pages/modify.php?page_id='.$page['page_id'];
                        $tree[$page['page_id']]['default_language']    = $default_language;
                        $tree[$page['page_id']]['pc_language']         = $sPageCodeLanguage;
    //                    
    //                    // Admin Groups (get user permissions)
                        $admin_groups = explode(',', str_replace('_', '', $page['admin_groups']));
                        $admin_users = explode(',', str_replace('_', '', $page['admin_users']));
                        $in_group = FALSE;
                        foreach($admin->get_groups_id() as $cur_gid) {
                            if (in_array($cur_gid, $admin_groups))     $in_group = TRUE;
                        }
                        // check modify permissions
                        $tree[$page['page_id']]['can_modify'] = FALSE;
                        if(($in_group) || is_numeric(array_search($admin->getUserId(), $admin_users))){
                            if($tree[$page['page_id']]['visibility'] == 'deleted'){
                                $tree[$page['page_id']]['can_modify'] = (PAGE_TRASH == 'inline') ? TRUE : FALSE;
                            } elseif ($tree[$page['page_id']]['visibility'] != 'deleted') {
                                $tree[$page['page_id']]['can_modify'] = TRUE;
                            }
                        } else {
                            if ($tree[$page['page_id']]['visibility'] == 'private') {
                                continue;
                            } else {
                                $tree[$page['page_id']]['can_modify'] = FALSE;
                            }
                        }

                        $tree[$page['page_id']]['canManageSections']  = (MANAGE_SECTIONS == 'enabled' && $admin->get_permission('pages_modify') == TRUE && $tree[$page['page_id']]['can_modify'] == TRUE) ? TRUE : FALSE;
                        $tree[$page['page_id']]['canModifyPage']      = ($admin->get_permission('pages_modify') == TRUE && $tree[$page['page_id']]['can_modify'] == TRUE) ? TRUE : FALSE;
                        $tree[$page['page_id']]['canModifySettings']  = ($admin->get_permission('pages_settings') == TRUE && $tree[$page['page_id']]['can_modify'] == TRUE) ? TRUE : FALSE;
                        $tree[$page['page_id']]['modifySettingsURL']  = '../pages/settings.php?page_id='.$page['page_id'];
                        $tree[$page['page_id']]['restoreURL']         = '../pages/restore.php?page_id='.$page['page_id'];

                        # sectionICON
                        $aSectionData = Pagecodecheck::IsPublished($page['page_id']);
                        $aLangPages['sectionICON'] = "noclock_16.png";
                        if($aSectionData['published'] != 0){
                            $tree[$page['page_id']]['sectionICON'] = ($admin->page_is_active($page)) ? "clock_16.png" : "clock_red_16.png";
                        }
                        if($aSectionData['module'] == 'menu_link') {
                            $tree[$page['page_id']]['sectionICON'] =  "menu_link_48.png";
                            $tree[$page['page_id']]['menu_link'] =  TRUE;
                        }
                        if ($tree[$page['page_id']]['canManageSections'] == TRUE){
                            $tree[$page['page_id']]['sectionsURL'] = '../pages/sections.php?page_id='.$page['page_id'];
                        }
                        if ($tree[$page['page_id']]['visibility'] == 'public') {
                            $tree[$page['page_id']]['visICON'] =  "visible_16";
                            $tree[$page['page_id']]['visTxt'] =  $oTrans->PCC_public ;
                        }elseif ($tree[$page['page_id']]['visibility'] == 'none') {
                            $tree[$page['page_id']]['visICON'] =  "none_16";
                            $tree[$page['page_id']]['visTxt'] =  $oTrans->PCC_none ;
                        }elseif ($tree[$page['page_id']]['visibility'] == 'hidden') {
                            $tree[$page['page_id']]['visICON'] =  "hidden_16";
                            $tree[$page['page_id']]['visTxt'] =  $oTrans->PCC_hidden ;
                        }elseif ($tree[$page['page_id']]['visibility'] == 'private') {
                            $tree[$page['page_id']]['visICON'] =  "private_16";
                            $tree[$page['page_id']]['visTxt'] =  $oTrans->PCC_private ;
                        }elseif ($tree[$page['page_id']]['visibility'] == 'registered') {
                            $tree[$page['page_id']]['visICON'] =  "keys_16";
                            $tree[$page['page_id']]['visTxt'] =  $oTrans->PCC_registered ;
                        }elseif ($tree[$page['page_id']]['visibility'] == 'deleted') {
                            $tree[$page['page_id']]['visICON'] =  "deleted_16";
                            $tree[$page['page_id']]['visTxt'] =  $oTrans->PCC_deleted ;
                        }
                        $tree = self::getPageTreeList($page['page_id'], $tree);
                    } // end while

    //                echo "<pre>";
    //                print_r($tree);
    //                echo "</pre>"; 
                return($tree);
            } 

        } // end getPageTreeList  

    

################################################################################
// count all Pagess (getSelectboxList)
###############################################################################
        static public function getSelectboxList(int $parent, $tree){
            $oReg = WbAdaptor::getInstance();
            $oDb = $oReg->getDataBase();
            global $oDb, $admin;    

            $aResult = [];
            $refs= [];
            $aLangPages = $aPages =  null;
            $default_language = DEFAULT_LANGUAGE;
            $sqlSet = 'SELECT `page_id`,`parent`,`root_parent`,`level`,`link`,`page_trail`,`position`, '
                    . '`language`,`visibility`,`page_title`,`menu_title`,`page_code`,`admin_groups`,`admin_users`'
                    . 'FROM `'.$oReg->TablePrefix.'pages`'
                    . 'WHERE `parent`='.(int)$parent.' '
                    . 'AND `language` =  \''.$default_language.'\'  '
                    . 'ORDER BY  `position`'
                    . '';   
    //      echo $sql."<br>";

                if ($query_page = $oDb->query($sqlSet)) {
                    while(!\is_null($page = $query_page->fetchRow(\MYSQLI_ASSOC))) {

                        $sPageCodeLanguage = Pagecodecheck::getPageCodeLanguage($page['page_code']);
                        $title_prefix = '';
                        for($i = 1; $i <= $page['level']; $i++) { $title_prefix .= ' - - &nbsp;'; }
                        $tree[$page['page_id']]['page_id']             = $page['page_id'];
                        $tree[$page['page_id']]['parent']              = $page['parent'];
                        $tree[$page['page_id']]['root_parent']         = $page['root_parent'];
                        $tree[$page['page_id']]['level']               = $page['level'];
                        $tree[$page['page_id']]['link']                = $page['link'];  //
    //                    $tree[$page['page_id']]['page_trail']          = $page['page_trail'];  //
                        $tree[$page['page_id']]['position']            = $page['position'];  //
                        $tree[$page['page_id']]['language']            = $page['language'];
                        $tree[$page['page_id']]['visibility']          = $page['visibility'];
                        $tree[$page['page_id']]['page_title']          = $page['page_title'];
                        $tree[$page['page_id']]['menu_title']          = $page['menu_title'];
                        $tree[$page['page_id']]['page_code']           = $page['page_code'];
                        $tree[$page['page_id']]['format_page_title']   = $title_prefix.$page['page_title'];
                        $tree[$page['page_id']]['default_language']    = $default_language;
                        $tree[$page['page_id']]['pc_language']         = $sPageCodeLanguage;
    //                    
    //                    // Admin Groups (get user permissions)
                        $admin_groups = explode(',', str_replace('_', '', $page['admin_groups']));
                        $admin_users = explode(',', str_replace('_', '', $page['admin_users']));
                        $in_group = FALSE;
                        foreach($admin->get_groups_id() as $cur_gid) {
                            if (in_array($cur_gid, $admin_groups))     $in_group = TRUE;
                        }
                        // check modify permissions
                        $tree[$page['page_id']]['can_modify'] = FALSE;
                        if(($in_group) || is_numeric(array_search($admin->getUserId(), $admin_users))){
                            if($tree[$page['page_id']]['visibility'] == 'deleted'){
                                $tree[$page['page_id']]['can_modify'] = (PAGE_TRASH == 'inline') ? TRUE : FALSE;
                            } elseif ($tree[$page['page_id']]['visibility'] != 'deleted') {
                                $tree[$page['page_id']]['can_modify'] = TRUE;
                            }
                        } else {
                            if ($tree[$page['page_id']]['visibility'] == 'private') {
                                continue;
                            } else {
                                $tree[$page['page_id']]['can_modify'] = FALSE;
                            }
                        }
                        $tree = self::getSelectboxList($page['page_id'], $tree);
                    } // end while

    //                echo "<pre>";
    //                print_r($tree);
    //                echo "</pre>"; 
                return($tree);
            } 

        } // end getSelectboxList     
    }
    
// end of file