<?php
/**
 * @category     Addons
 * @package      Admin tool: Pagescode-Check
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Org. e.V.
 * @author       Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Project
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.0.0
 * @lastmodified 2024/10/06
 * @since        File available since 2024/10/06
 * @description  info for wbpagecode
 *
 * function.lib
*/
namespace addon\wbPagesec\Lib;

use bin\{WbAdaptor, SecureTokens};
use bin\helpers\{PreCheck,StopWatch};

#[AllowDynamicProperties]

    class Pagesec {

    /** @var object instance of the application object */
        protected $_oApp;
    /** @var object instance of the database object */
        protected $_oDb ;
    /** @var object instance holds several values from the application global scope */
        protected $_oReg;
    /** @var object instance holds all of the translations */
        protected $_oTrans;


    // -----------------------------------------------------------------------------
    /** prevent class from public instancing and get an object to hold extensions */
    public function __construct()
    {
        // import global vars and objects
        $this->_oReg   = WbAdaptor::getInstance();
        $this->_oDb    = $this->_oReg->getDatabase();
        $this->_oTrans = $this->_oReg->getTranslate();
        $this->_oApp   = $this->_oReg->getApplication();
    }
    // -----------------------------------------------------------------------------
    public function getSettings($iSectionId) {
        return $this->_readSettings($iSectionId);
    }
    // -----------------------------------------------------------------------------
    
    ###############################################################################
        // read settings for this section (_readSettings)
        ###############################################################################
        /** read default setting for this section */
                 protected function _readSettings($iSectionId){
                    global $aDefaults, $oAddonReg, $section_id, $page_id, $sAddonPath ;
//                    $oDb = \database::getInstance();
                        $aPageSecSettings  = [];
                        $sql  = 'SELECT `s_name`, `s_value` FROM `'. $this->_oDb->TablePrefix .'mod_pagesec_settings` '
                              . 'WHERE `section_id` = ' . $iSectionId;
//                        echo $sql;
                        if ($query = $this->_oDb->query($sql)){
                            while ($row = $query->fetchAssoc()) {
                                $aPageSecSettings[$row['s_name']] = $row['s_value'];
                            }
                        }

                        // insert default_settings if no settings for this section avaiable
                        if (sizeof($aPageSecSettings) == 0){
                            $aFieldsList = array(
                                'section_id' => 0,
                                'page_id'    => 0,
                            );
                            /* add default setting if not set for this section */
                            if (\is_array($aDefaults)) {
                                $aTmp = \array_diff_key($aDefaults, $aPageSecSettings);
                                if ($aTmp){
                                    UpdateKeyValue('mod_pagesec_settings', $aDefaults, '', $aFieldsList);
                                }
                            }
                        }
//                echo "<pre>";
//                print_r($aPageSecSettings);
//                echo "</pre>";    
                return $aPageSecSettings;
                }
        // end of static public function ('_readSettings')
    
    
    
    ################################################################################
    // build mudulelist (getModules)
    ###############################################################################    
    static public function moduleList() {
        global $database, $admin;
        $sAddonFunction = 'page';        
        $sql  = 'SELECT `directory`,`function` FROM `'.TABLE_PREFIX.'addons` '
              . 'WHERE `function`=\''.$sAddonFunction.'\' '
              . 'ORDER BY `name`'
              . '';
//        echo $sql;
        if (!$oAddons = $database->query($sql)) {
            throw new \Exception ($database->get_error());
        }
        while($aAddons = $oAddons->fetchAssoc()) {
            $aAddon['directory'] = $aAddons['directory'];
            $aResult[] = $aAddon;
        }
        #echo "<pre>";
        #print_r($aResult);
        #echo "</pre>";
        // return the complete result structure
        return $aResult;
    }
    ################################################################################
    // build list of all used page-id's (usedPagesList)
    ###############################################################################    
    static public function usedPagesList() {
        global $database, $admin;
        $sAddonFunction = 'page';        
        $sql  = 'SELECT `page_id`,`language`,`page_title` FROM `'.TABLE_PREFIX.'pages` '
              . 'ORDER BY `page_id`,`language`'
              . '';
//        echo $sql;
        if (!$oPages = $database->query($sql)) {
            throw new \Exception ($database->get_error());
        }
        while($aPages = $oPages->fetchAssoc()) {
            $aPage['page_id'] = $aPages['page_id'];
            $aPage['page_title'] = $aPages['page_title'];
            $aResult[] = $aPage;
        }
        #echo "<pre>";
        #print_r($aResult);
        #echo "</pre>";
        // return the complete result structure
        return $aResult;
    } 
        ################################################################################
    // build list of all used section-id's (usedSectionList)
    ###############################################################################    
    static public function usedSectionList() {
        global $database, $admin;
        $sAddonFunction = 'page';        
        $sql  = 'SELECT `page_id`,`section_id`,`module` FROM `'.TABLE_PREFIX.'sections` '
              . 'ORDER BY `page_id`,`module`'
              . '';
//        echo $sql;
        if (!$oSections = $database->query($sql)) {
            throw new \Exception ($database->get_error());
        }
        while($aSections = $oSections->fetchAssoc()) {
            $aSection['page_id']       = $aSections['page_id'];
            $aSection['section_id']    = $aSections['section_id'];
            $aSection['module']        = $aSections['module'];
            $aResult[] = $aSection;
        }
//        echo "<pre>";
//        print_r($aResult);
//        echo "</pre>";
        // return the complete result structure
        return $aResult;
    }
    ################################################################################
    // build PagesList (getPages)
    ###############################################################################
    static public function pagesArray($bProcessSeo = FALSE) {
        global $database, $admin;
        // prepare SQL Query, build the query string first
        $sUseTrash = (PAGE_TRASH != 'inline') ? " WHERE `visibility` <> 'deleted'" : '';
        $sProcessSeo = ($bProcessSeo == TRUE) ? ', p.`description`, p.`keywords` ' : ' ';
        $sProcessRewriteUrl = '';
        $bRewriteUrlExists = FALSE;
        if(defined("REWRITE_URL") == TRUE && REWRITE_URL != ''){
            $oCheckDbTable = $database->query("SHOW COLUMNS FROM `".TABLE_PREFIX."pages` LIKE '".REWRITE_URL."'");
            $bRewriteUrlExists = $oCheckDbTable->numRows() > 0 ? TRUE : FALSE;
            $sProcessRewriteUrl = ($bRewriteUrlExists == TRUE) ? ' p.`'.REWRITE_URL.'`,' : '';
        }

         $sQuery = 'SELECT  s.`module`, MAX(s.`publ_start` + s.`publ_end`) published, p.`link`, '
             .        '(SELECT MAX(`position`) FROM `'. TABLE_PREFIX .'pages` WHERE `parent`=p.`parent`) siblings, '
             .        'p.`position`, p.`page_id`, p.`parent`, p.`level`, p.`language`, p.`admin_groups`, '
             .    (($bRewriteUrlExists == TRUE ) ? 'p.`'.REWRITE_URL.'`, ' : '' ) .''
             .        'p.`admin_users`, p.`viewing_groups`, p.`viewing_users`, p.`visibility`, '
             .        'p.`menu_title`, p.`page_title`, p.`page_trail`'.$sProcessSeo.''
             . 'FROM `'. TABLE_PREFIX .'pages` p '
             .    'INNER JOIN `'. TABLE_PREFIX .'sections` s '
             .    'ON p.`page_id`=s.`page_id` '
             . $sUseTrash.' '
             . 'GROUP BY p.`page_id`, s.`module` '
             . 'ORDER BY p.`position` ASC';

        $oPages = $database->query($sQuery);

        // echo $database->get_error();

        $aPages = array();
        $refs = array();
        // create $aPages[] Array
        while($page = $oPages->fetchAssoc()) {
            $thisref = &$refs[ $page['page_id'] ];
            $thisref['page_id']        = $page['page_id'];
            $thisref['pageIDKEY']      = (method_exists($admin, "getIDKEY")) ? $admin->getIDKEY($page['page_id']) : $page['page_id'];
            $thisref['root_parent']    = isset( $page['root_parent'] ) ? $page['root_parent'] : ''; // is this 'root_parent' needed at all?
            $thisref['language']       = $page['language'];
            $thisref['siblings']       = $page['siblings'];
            $thisref['position']       = $page['position'];
            $thisref['parent']         = $page['parent'];
            $thisref['menu_title']     = $page['menu_title'];
            $thisref['page_title']     = $page['page_title'];
            $thisref['level']          = $page['level'];
            $thisref['visibility']     = $page['visibility'];
            $thisref['admin_groups']   = $page['admin_groups'];
            $thisref['admin_users']    = $page['admin_users'];
            $thisref['position']       = $page['position'];
            if($bProcessSeo == TRUE) {
                $thisref['description']  = $page['description'];
                $thisref['keywords']     = $page['keywords'];
            }
            if(($bRewriteUrlExists == TRUE)) {
                $thisref[REWRITE_URL]  = $page[REWRITE_URL];
            }
            $thisref['frontend_link']  = PAGES_DIRECTORY . $page['link'] . PAGE_EXTENSION;
            $thisref['modify_page_url']= '../pages/modify.php?page_id='.$page['page_id'];

            // Admin Groups (get user permissions)
            $admin_groups = explode(',', str_replace('_', '', $page['admin_groups']));
            $admin_users = explode(',', str_replace('_', '', $page['admin_users']));
            $in_group = FALSE;
            foreach($admin->get_groups_id() as $cur_gid) {
                if (in_array($cur_gid, $admin_groups))     $in_group = TRUE;
            }
            // check modify permissions
            $thisref['can_modify'] = FALSE;
            if(($in_group) || is_numeric(array_search($admin->getUserId(), $admin_users))){
                if($page['visibility'] == 'deleted'){
                    $thisref['can_modify'] = (PAGE_TRASH == 'inline') ? TRUE : FALSE;
                } elseif ($page['visibility'] != 'deleted') {
                    $thisref['can_modify'] = TRUE;
                }
            } else {
                if ($page['visibility'] == 'private') {
                    continue;
                } else {
                    $thisref['can_modify'] = FALSE;
                }
            }
            $thisref['canManageSections'] = (MANAGE_SECTIONS == 'enabled' && $admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($thisref['page_movable'] && $thisref['siblings'] != 1) ? TRUE : FALSE;
            $thisref['canMoveUP']          = ($thisref['page_movable'] && $thisref['position'] != 1) ? TRUE : FALSE;
            $thisref['canMoveDOWN']        = ($thisref['page_movable'] && $thisref['position'] != $thisref['siblings']) ? TRUE : FALSE;
            $thisref['canDeleteAndModify'] = ($admin->get_permission('pages_delete') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canAddChild']        = ($admin->get_permission('pages_add')) == (TRUE && $thisref['can_modify'] == TRUE) && ($thisref['visibility'] != 'deleted') ? TRUE : FALSE;
            $thisref['canModifyPage']      = ($admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canModifySettings']  = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['modifySettingsURL']  = '../pages/settings.php?page_id='.$page['page_id'];
            $thisref['restoreURL']         = '../pages/restore.php?page_id='.$page['page_id'];

            # sectionICON
            $thisref['sectionICON'] = "noclock_16.png";
            if($page['published'] != 0){
                $thisref['sectionICON'] = ($admin->page_is_active($thisref)) ? "clock_16.png" : "clock_red_16.png";
            }
            if($page['module'] == 'menu_link') {
                $thisref['sectionICON'] =  "menu_link_48.png";
                $thisref['menu_link'] =  TRUE;
            }
            if ($thisref['canManageSections'] == TRUE){
                $thisref['sectionsURL'] = '../pages/sections.php?page_id='.$thisref['page_id'];
            }

            if ($page['parent'] == 0) {
                $aPages[ $page['page_id'] ] = &$thisref;
            } else {
                $refs[ $page['parent'] ]['children'][ $page['page_id'] ] = &$thisref;
            }
            unset($page);
        }
        
//                echo "<pre>";
//                print_r($aPages);
//                echo "</pre>";
        return $aPages;
    } // end of static public function pagesarray
    
    ################################################################################
    // search for a special page
    ###############################################################################
    static public function pagesSearch($iSearchPageId, $bProcessSeo = FALSE) {
        global $database, $admin;
        // prepare SQL Query, build the query string first
        $sUseTrash = (PAGE_TRASH != 'inline') ? " AND `visibility` <> 'deleted'" : '';
        $sProcessSeo = ($bProcessSeo == TRUE) ? ', p.`description`, p.`keywords` ' : ' ';
        $sProcessRewriteUrl = '';
        $bRewriteUrlExists = FALSE;
        if(defined("REWRITE_URL") == TRUE && REWRITE_URL != ''){
            $oCheckDbTable = $database->query("SHOW COLUMNS FROM `".TABLE_PREFIX."pages` LIKE '".REWRITE_URL."'");
            $bRewriteUrlExists = $oCheckDbTable->numRows() > 0 ? TRUE : FALSE;
            $sProcessRewriteUrl = ($bRewriteUrlExists == TRUE) ? ' p.`'.REWRITE_URL.'`,' : '';
        }

         $sQuery = 'SELECT  s.`module`, MAX(s.`publ_start` + s.`publ_end`) published, p.`link`, '
             .        '(SELECT MAX(`position`) FROM `'. TABLE_PREFIX .'pages` WHERE `parent`=p.`parent`) siblings, '
             .        'p.`position`, p.`page_id`, p.`parent`, p.`level`, p.`language`, p.`admin_groups`, '
             .    (($bRewriteUrlExists == TRUE ) ? 'p.`'.REWRITE_URL.'`, ' : '' ) .''
             .        'p.`admin_users`, p.`viewing_groups`, p.`viewing_users`, p.`visibility`, '
             .        'p.`menu_title`, p.`page_title`, p.`page_trail`'.$sProcessSeo.''
             . 'FROM `'. TABLE_PREFIX .'pages` p '
             .    'INNER JOIN `'. TABLE_PREFIX .'sections` s '
             .    'ON p.`page_id`=s.`page_id` '
             . 'WHERE p.`page_id` = '.(int)$iSearchPageId.' '    
             . $sUseTrash.' '
             . 'GROUP BY p.`page_id`, s.`module` '
             . 'ORDER BY p.`position` ASC';
//        echo $sQuery;
        $oPages = $database->query($sQuery);

        // echo $database->get_error();

        $aPages = [];
        $refs = [];
        // create $aPages[] Array
        while($page = $oPages->fetchAssoc()) {
            $thisref = &$refs[ $page['page_id'] ];
            $thisref['page_id']        = $page['page_id'];
            $thisref['pageIDKEY']      = (method_exists($admin, "getIDKEY")) ? $admin->getIDKEY($page['page_id']) : $page['page_id'];
            $thisref['root_parent']    = isset( $page['root_parent'] ) ? $page['root_parent'] : ''; // is this 'root_parent' needed at all?
            $thisref['language']       = $page['language'];
            $thisref['siblings']       = $page['siblings'];
            $thisref['position']       = $page['position'];
            $thisref['parent']         = $page['parent'];
            $thisref['menu_title']     = $page['menu_title'];
            $thisref['page_title']     = $page['page_title'];
            $thisref['level']          = $page['level'];
            $thisref['visibility']     = $page['visibility'];
            $thisref['admin_groups']   = $page['admin_groups'];
            $thisref['admin_users']    = $page['admin_users'];
            $thisref['position']       = $page['position'];
            if($bProcessSeo == TRUE) {
                $thisref['description']  = $page['description'];
                $thisref['keywords']     = $page['keywords'];
            }
            if(($bRewriteUrlExists == TRUE)) {
                $thisref[REWRITE_URL]  = $page[REWRITE_URL];
            }
            $thisref['frontend_link']  = PAGES_DIRECTORY . $page['link'] . PAGE_EXTENSION;
            $thisref['modify_page_url']= '../pages/modify.php?page_id='.$page['page_id'];

            // Admin Groups (get user permissions)
            $admin_groups = explode(',', str_replace('_', '', $page['admin_groups']));
            $admin_users = explode(',', str_replace('_', '', $page['admin_users']));
            $in_group = FALSE;
            foreach($admin->get_groups_id() as $cur_gid) {
                if (in_array($cur_gid, $admin_groups))     $in_group = TRUE;
            }
            // check modify permissions
            $thisref['can_modify'] = FALSE;
            if(($in_group) || is_numeric(array_search($admin->getUserId(), $admin_users))){
                if($page['visibility'] == 'deleted'){
                    $thisref['can_modify'] = (PAGE_TRASH == 'inline') ? TRUE : FALSE;
                } elseif ($page['visibility'] != 'deleted') {
                    $thisref['can_modify'] = TRUE;
                }
            } else {
                if ($page['visibility'] == 'private') {
                    continue;
                } else {
                    $thisref['can_modify'] = FALSE;
                }
            }
            $thisref['canManageSections'] = (MANAGE_SECTIONS == 'enabled' && $admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($thisref['page_movable'] && $thisref['siblings'] != 1) ? TRUE : FALSE;
            $thisref['canMoveUP']          = ($thisref['page_movable'] && $thisref['position'] != 1) ? TRUE : FALSE;
            $thisref['canMoveDOWN']        = ($thisref['page_movable'] && $thisref['position'] != $thisref['siblings']) ? TRUE : FALSE;
            $thisref['canDeleteAndModify'] = ($admin->get_permission('pages_delete') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canAddChild']        = ($admin->get_permission('pages_add')) == (TRUE && $thisref['can_modify'] == TRUE) && ($thisref['visibility'] != 'deleted') ? TRUE : FALSE;
            $thisref['canModifyPage']      = ($admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canModifySettings']  = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['modifySettingsURL']  = '../pages/settings.php?page_id='.$page['page_id'];
            $thisref['restoreURL']         = '../pages/restore.php?page_id='.$page['page_id'];

            # sectionICON
            $thisref['sectionICON'] = "noclock_16.png";
            if($page['published'] != 0){
                $thisref['sectionICON'] = ($admin->page_is_active($thisref)) ? "clock_16.png" : "clock_red_16.png";
            }
            if($page['module'] == 'menu_link') {
                $thisref['sectionICON'] =  "menu_link_48.png";
                $thisref['menu_link'] =  TRUE;
            }
            if ($thisref['canManageSections'] == TRUE){
                $thisref['sectionsURL'] = '../pages/sections.php?page_id='.$thisref['page_id'];
            }
$aPages[ $page['page_id'] ] = &$thisref;
//            if ($page['parent'] == 0) {
//                $aPages[ $page['page_id'] ] = &$thisref;
//            } else {
//                $refs[ $page['parent'] ]['children'][ $page['page_id'] ] = &$thisref;
//            }
            unset($page);
            
            
            
        }
        
//                echo "<pre>";
//                print_r($aPages);
//                echo "</pre>";
        return $aPages;
    } // end of static public function pagessearch
    
    
    ################################################################################
    // search for a special page - moduleSearch
    ###############################################################################
    static public function moduleSearch($sModuleName, $bProcessSeo = TRUE) {
        global $database, $admin;
        // prepare SQL Query, build the query string first
        $sUseTrash = (PAGE_TRASH != 'inline') ? " AND `visibility` <> 'deleted'" : '';
        $sProcessSeo = ($bProcessSeo == TRUE) ? ', p.`description`, p.`keywords` ' : ' ';
        $sProcessRewriteUrl = '';
        $bRewriteUrlExists = FALSE;
        if(defined("REWRITE_URL") == TRUE && REWRITE_URL != ''){
            $oCheckDbTable = $database->query("SHOW COLUMNS FROM `".TABLE_PREFIX."pages` LIKE '".REWRITE_URL."'");
            $bRewriteUrlExists = $oCheckDbTable->numRows() > 0 ? TRUE : FALSE;
            $sProcessRewriteUrl = ($bRewriteUrlExists == TRUE) ? ' p.`'.REWRITE_URL.'`,' : '';
        }

         $sQuery = 'SELECT  s.`module`, MAX(s.`publ_start` + s.`publ_end`) published, p.`link`, '
             .        '(SELECT MAX(`position`) FROM `'. TABLE_PREFIX .'pages` WHERE `parent`=p.`parent`) siblings, '
             .        'p.`position`, p.`page_id`, p.`parent`, p.`level`, p.`language`, p.`admin_groups`, '
             .    (($bRewriteUrlExists == TRUE ) ? 'p.`'.REWRITE_URL.'`, ' : '' ) .''
             .        'p.`admin_users`, p.`viewing_groups`, p.`viewing_users`, p.`visibility`, '
             .        'p.`menu_title`, p.`page_title`, p.`page_trail`'.$sProcessSeo.''
             . 'FROM `'. TABLE_PREFIX .'pages` p '
             .    'INNER JOIN `'. TABLE_PREFIX .'sections` s '
             .    'ON p.`page_id`=s.`page_id` '
             . 'WHERE s.`module` LIKE \''.$sModuleName.'\' '    
             . $sUseTrash.' '
             . 'GROUP BY p.`page_id`, s.`module` '
             . 'ORDER BY p.`page_id` ASC';
//        echo $sQuery;
        $oPages = $database->query($sQuery);

        // echo $database->get_error();

        $aPages = array();
        $refs = array();
        // create $aPages[] Array
        while($page = $oPages->fetchAssoc()) {
            
            $thisref = &$refs[ $page['page_id'] ];
            $thisref['page_id']        = $page['page_id'];
            $thisref['pageIDKEY']      = (method_exists($admin, "getIDKEY")) ? $admin->getIDKEY($page['page_id']) : $page['page_id'];
            $thisref['root_parent']    = isset( $page['root_parent'] ) ? $page['root_parent'] : ''; // is this 'root_parent' needed at all?
            $thisref['language']       = $page['language'];
            $thisref['siblings']       = $page['siblings'];
            $thisref['position']       = $page['position'];
            $thisref['parent']         = $page['parent'];
            $thisref['menu_title']     = $page['menu_title'];
            $thisref['page_title']     = $page['page_title'];
            $thisref['level']          = $page['level'];
            $thisref['visibility']     = $page['visibility'];
            $thisref['admin_groups']   = $page['admin_groups'];
            $thisref['admin_users']    = $page['admin_users'];
            $thisref['position']       = $page['position'];
            if($bProcessSeo == TRUE) {
                $thisref['description']  = $page['description'];
                $thisref['keywords']     = $page['keywords'];
            }
            if(($bRewriteUrlExists == TRUE)) {
                $thisref[REWRITE_URL]  = $page[REWRITE_URL];
            }
            $thisref['frontend_link']  = PAGES_DIRECTORY . $page['link'] . PAGE_EXTENSION;
            $thisref['modify_page_url']= '../pages/modify.php?page_id='.$page['page_id'];

            // Admin Groups (get user permissions)
            $admin_groups = explode(',', str_replace('_', '', $page['admin_groups']));
            $admin_users = explode(',', str_replace('_', '', $page['admin_users']));
            $in_group = FALSE;
            foreach($admin->get_groups_id() as $cur_gid) {
                if (in_array($cur_gid, $admin_groups))     $in_group = TRUE;
            }
            // check modify permissions
            $thisref['can_modify'] = FALSE;
            if(($in_group) || is_numeric(array_search($admin->getUserId(), $admin_users))){
                if($page['visibility'] == 'deleted'){
                    $thisref['can_modify'] = (PAGE_TRASH == 'inline') ? TRUE : FALSE;
                } elseif ($page['visibility'] != 'deleted') {
                    $thisref['can_modify'] = TRUE;
                }
            } else {
                if ($page['visibility'] == 'private') {
                    continue;
                } else {
                    $thisref['can_modify'] = FALSE;
                }
            }
            $thisref['canManageSections'] = (MANAGE_SECTIONS == 'enabled' && $admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($thisref['page_movable'] && $thisref['siblings'] != 1) ? TRUE : FALSE;
            $thisref['canMoveUP']          = ($thisref['page_movable'] && $thisref['position'] != 1) ? TRUE : FALSE;
            $thisref['canMoveDOWN']        = ($thisref['page_movable'] && $thisref['position'] != $thisref['siblings']) ? TRUE : FALSE;
            $thisref['canDeleteAndModify'] = ($admin->get_permission('pages_delete') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canAddChild']        = ($admin->get_permission('pages_add')) == (TRUE && $thisref['can_modify'] == TRUE) && ($thisref['visibility'] != 'deleted') ? TRUE : FALSE;
            $thisref['canModifyPage']      = ($admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canModifySettings']  = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['modifySettingsURL']  = '../pages/settings.php?page_id='.$page['page_id'];
            $thisref['restoreURL']         = '../pages/restore.php?page_id='.$page['page_id'];

            # sectionICON
            $thisref['sectionICON'] = "noclock_16.png";
            if($page['published'] != 0){
                $thisref['sectionICON'] = ($admin->page_is_active($thisref)) ? "clock_16.png" : "clock_red_16.png";
            }
            if($page['module'] == 'menu_link') {
                $thisref['sectionICON'] =  "menu_link_48.png";
                $thisref['menu_link'] =  TRUE;
            }
            if ($thisref['canManageSections'] == TRUE){
                $thisref['sectionsURL'] = '../pages/sections.php?page_id='.$thisref['page_id'];
            }


                $aPages[ $page['page_id'] ] = &$thisref;

            unset($page);
        }
        
//                echo "<pre>";
//                print_r($aPages);
//                echo "</pre>";
        return $aPages;
    } // end of static public function moduleSearch   
    
    
    ################################################################################
    // search for a special sections
    ###############################################################################
    static public function sectionSearch($sModuleName, $bProcessSeo = FALSE) {
        global $database, $admin;
        // prepare SQL Query, build the query string first
        $sUseTrash = (PAGE_TRASH != 'inline') ? " AND `visibility` <> 'deleted'" : '';
        $sProcessSeo = ($bProcessSeo == TRUE) ? ', p.`description`, p.`keywords` ' : ' ';
        $sProcessRewriteUrl = '';
        $bRewriteUrlExists = FALSE;
        if(defined("REWRITE_URL") == TRUE && REWRITE_URL != ''){
            $oCheckDbTable = $database->query("SHOW COLUMNS FROM `".TABLE_PREFIX."pages` LIKE '".REWRITE_URL."'");
            $bRewriteUrlExists = $oCheckDbTable->numRows() > 0 ? TRUE : FALSE;
            $sProcessRewriteUrl = ($bRewriteUrlExists == TRUE) ? ' p.`'.REWRITE_URL.'`,' : '';
        }

        $sQuery =  'SELECT `s`.*,`p`.* '
                .  'FROM `'.TABLE_PREFIX.'sections` `s` '
                .  'LEFT OUTER JOIN `'.TABLE_PREFIX.'pages` `p` ON `p`.`page_id` = `s`.`page_id` '
                .  'WHERE `s`.`module` LIKE \''.$sModuleName.'\' '
                .  'ORDER BY `p`.`page_id`';
        $oPages = $database->query($sQuery);

        // echo $database->get_error();

        $aPages = [];
        $refs = [];
        // create $aPages[] Array
        while($page = $oPages->fetchAssoc()) {
            $thisref = &$refs[ $page['page_id'] ];
            $thisref['page_id']        = $page['page_id'];
            $thisref['pageIDKEY']      = (method_exists($admin, "getIDKEY")) ? $admin->getIDKEY($page['page_id']) : $page['page_id'];
            $thisref['root_parent']    = isset( $page['root_parent'] ) ? $page['root_parent'] : ''; // is this 'root_parent' needed at all?
            $thisref['language']       = $page['language'];
//            $thisref['siblings']       = $page['siblings'];
            $thisref['position']       = $page['position'];
            $thisref['parent']         = $page['parent'];
            $thisref['menu_title']     = $page['menu_title'];
            $thisref['page_title']     = $page['page_title'];
            $thisref['level']          = $page['level'];
            $thisref['visibility']     = $page['visibility'];
            $thisref['admin_groups']   = $page['admin_groups'];
            $thisref['admin_users']    = $page['admin_users'];
            $thisref['position']       = $page['position'];
            if($bProcessSeo == TRUE) {
                $thisref['description']  = $page['description'];
                $thisref['keywords']     = $page['keywords'];
            }
            if(($bRewriteUrlExists == TRUE)) {
                $thisref[REWRITE_URL]  = $page[REWRITE_URL];
            }
            $thisref['frontend_link']  = PAGES_DIRECTORY . $page['link'] . PAGE_EXTENSION;
            $thisref['modify_page_url']= '../pages/modify.php?page_id='.$page['page_id'];

            // Admin Groups (get user permissions)
            $admin_groups = explode(',', str_replace('_', '', $page['admin_groups']));
            $admin_users = explode(',', str_replace('_', '', $page['admin_users']));
            $in_group = FALSE;
            foreach($admin->get_groups_id() as $cur_gid) {
                if (in_array($cur_gid, $admin_groups))     $in_group = TRUE;
            }
            // check modify permissions
            $thisref['can_modify'] = FALSE;
            if(($in_group) || is_numeric(array_search($admin->getUserId(), $admin_users))){
                if($page['visibility'] == 'deleted'){
                    $thisref['can_modify'] = (PAGE_TRASH == 'inline') ? TRUE : FALSE;
                } elseif ($page['visibility'] != 'deleted') {
                    $thisref['can_modify'] = TRUE;
                }
            } else {
                if ($page['visibility'] == 'private') {
                    continue;
                } else {
                    $thisref['can_modify'] = FALSE;
                }
            }
            $thisref['canManageSections'] = (MANAGE_SECTIONS == 'enabled' && $admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['page_movable']       = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
//            $thisref['page_movable']       = ($thisref['page_movable'] && $thisref['siblings'] != 1) ? TRUE : FALSE;
            $thisref['canMoveUP']          = ($thisref['page_movable'] && $thisref['position'] != 1) ? TRUE : FALSE;
//            $thisref['canMoveDOWN']        = ($thisref['page_movable'] && $thisref['position'] != $thisref['siblings']) ? TRUE : FALSE;
            $thisref['canDeleteAndModify'] = ($admin->get_permission('pages_delete') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canAddChild']        = ($admin->get_permission('pages_add')) == (TRUE && $thisref['can_modify'] == TRUE) && ($thisref['visibility'] != 'deleted') ? TRUE : FALSE;
            $thisref['canModifyPage']      = ($admin->get_permission('pages_modify') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['canModifySettings']  = ($admin->get_permission('pages_settings') == TRUE && $thisref['can_modify'] == TRUE) ? TRUE : FALSE;
            $thisref['modifySettingsURL']  = '../pages/settings.php?page_id='.$page['page_id'];
            $thisref['restoreURL']         = '../pages/restore.php?page_id='.$page['page_id'];

            # sectionICON
            $thisref['sectionICON'] = "noclock_16.png";
            if($page['published'] != 0){
                $thisref['sectionICON'] = ($admin->page_is_active($thisref)) ? "clock_16.png" : "clock_red_16.png";
            }
            if($page['module'] == 'menu_link') {
                $thisref['sectionICON'] =  "menu_link_48.png";
                $thisref['menu_link'] =  TRUE;
            }
            if ($thisref['canManageSections'] == TRUE){
                $thisref['sectionsURL'] = '../pages/sections.php?page_id='.$thisref['page_id'];
            }

            if ($page['parent'] == 0) {
                $aPages[ $page['page_id'] ] = &$thisref;
            } else {
                $refs[ $page['parent'] ]['children'][ $page['page_id'] ] = &$thisref;
            }
            unset($page);
            
            
            
        }
        
//                echo "<pre>";
//                print_r($aPages);
//                echo "</pre>";
        return $aPages;
    } // end of static public function sectionSearch
}

            
    
    
// end of file