<?php
/**
 * @category     Addons
 * @package      Admin tool: Pagescode-Check
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Org. e.V.
 * @author       Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Project
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.0.2
 * @lastmodified 2024/10/06
 * @since        File available since 2024/10/06
 * @description  info for wbpagecode
 *
/**
 * Install.inc.php
/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
if(defined('WB_PATH') == false) { die('Illegale file access /'.basename(__DIR__).'/'.basename(__FILE__).''); }
/* -------------------------------------------------------- */

use addon\wbpagecode\Lib\Pagecodecheck;
use bin\{WbAdaptor,SecureTokens};
use bin\helpers\{PreCheck};

$msg = '';
$sErrorMsg = null;
$sAddonPath = \str_replace('\\', '/', dirname(__DIR__));
$sAddonName = basename($sAddonPath);
$globalStarted = preg_match('/upgrade\-script\.php$/', $_SERVER["SCRIPT_NAME"]);
$sWbVersion = ($globalStarted && defined('VERSION') ? VERSION : WB_VERSION);
$sModulePlatform = PreCheck::getAddonVariable($sAddonName, 'platform');
if (\version_compare($sWbVersion, $sModulePlatform, '<')) {
    $msg = $sErrorMsg = \sprintf('It is not possible to install from WebsiteBaker Versions before %s', $sModulePlatform) . PHP_EOL;
    if ($globalStarted) {
        echo $sErrorMsg . PHP_EOL;
    } else {
        throw new \Exception($sErrorMsg);
    }
} else {
    Translate::getInstance()->disableAddon('modules\\' . $sAddonName);
    Translate::getInstance()->enableAddon(ADMIN_DIRECTORY . '\\addons');
    if (\is_writable(WB_PATH . '/temp/cache')) {
        Translate::getInstance()->clearCache();
    }
    $sTable = TABLE_PREFIX . 'mod_pagecode_settings';
    $i = (!isset($i) ? 1 : $i);
    $OK = "<span class=\"ok\">OK</span>";
    $FAIL = "<span class=\"error\">FAILED</span>";
    $iErr = false;
    $msg .= '<div style="margin:1em auto;font-size:1.1em;">' . PHP_EOL;
    $msg .= '<h4>Step ' . $i++ . ': Installing PagecodeCheck Settings</h4>' . PHP_EOL;

    // create tables from sql dump file
    $sInstallStruct = $sAddonPath . '/install-struct.sql.php';
    if (!\is_readable($sInstallStruct)) {
        $sInstallStruct = $sAddonPath . '/install-struct.sql.php';
    }
    if (!\is_readable($sInstallStruct)) {
        $msg .= \sprintf('<strong>missing or not readable file [%s]</strong> %s<br> ', basename($sInstallStruct), $FAIL);
        $iErr = true;
    } else {
        $aDefaults = [
            'section_id' => '0',
            'page_id' => '0',
            'iTitleCount_use' => '1',
            'iTitleCount_minimum' => '30',
            'iTitleCount_optimum' => '50',
            'iDescriptionCount_use' => '1',
            'iDescriptionCount_minimum' => '90',
            'iDescriptionCount_optimum' => '150',
            'keywordsConfig_use' => '1',
            'keywordsConfig_wordReplace' => 'keywords',
            'bUseRemainingChars_use' => '1',
            'bUseFlags_use' => '1',
            'rewriteUrl_use' => '1',
            'rewriteUrl_dbString' => 'custom01',
            'bUseRemainingChars' => '1',
        ];

        if ($database->SqlImport($sInstallStruct, TABLE_PREFIX, 'install')) {
            if (count($aDefaults)) {
                // restore old settings if there any
                $sNameValPairs = '';
                foreach ($aDefaults as $index => $val) {
                    $sNameValPairs .= ', (\'' . $index . '\', \'' . $database->escapeString($val) . '\')';
                }
                $sValues = ltrim($sNameValPairs, ', ');
                $sql = 'INSERT INTO `' . $sTable . '` (`s_name`, `s_value`) '
                        . 'VALUES ' . $sValues;
                if (!$database->query($sql)) {
                    $msg .= '<strong>PagecodeCheck install settings</strong> ' . $FAIL . PHP_EOL;
                    $msg .= $database->get_error() . '';
                    $iErr = true;
                } else {
                    $msg .= '<strong>PagecodeCheck settings successful installed</strong> ' . $OK . PHP_EOL;
                }
            }
            unset($getDefaultSettings);
        } else {
            $msg .= '<strong>PagecodeCheck install settings</strong> ' . $FAIL . PHP_EOL;
            $msg .= $database->get_error() . PHP_EOL;
        }
    }
    $msg .= '</div>' . PHP_EOL;
    if ($globalStarted) {
        echo nl2br($msg) . "<br>";
    }
}
// end of file

