<?php
/**
 * @category     Addons
 * @package      Admin tool: Pagescode-Check
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Org. e.V.
 * @author       Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Project
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.0.2
 * @lastmodified 2024/10/06
 * @since        File available since 2024/10/06
 * @description  info for wbpagecode
 *
/**
 * Upgrade.inc.php
 */



use bin\{WbAdaptor,SecureTokens,Sanitize};
use bin\helpers\{PreCheck};
// prevent this file from being accessed directly
/* -------------------------------------------------------- */
// Must include code to prevent this file from being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; flush(); exit; }
/* -------------------------------------------------------- */
/*
 * BEGIN: Create new settings table
 */

    $sAddonPath = str_replace('\\','/',dirname(__DIR__)).'/';
    echo $sAddonPath;
    $sAddonName = \basename($sAddonPath);
    $aMsg = [];
    $sMsg = '';
    $iErr = false;
    $bLoaded = null;

try {
    $oReg = WbAdaptor::getInstance();
/**
 * change update.fixed to update
 * if any template or javascript fixed in templates/default/ folder
 */
     $sTemplateFile = 'none';
     $sUpgradeFile = $sAddonPath.'update';
     $isUpgradeFile = $sUpgradeFile.'.fixed';
     if (\is_readable($sUpgradeFile)){
        if (!\rename($sUpgradeFile, $sUpgradeFile.'.fixed')){
            echo $sMsg = \sprintf('renaming of %s %s <br>',\basename($sUpgradeFile.'.fixed'), 'FAILED');
        } elseif(!$isUpgradeFile) {
            $sTemplateFile = ($oReg->Template ?? 'none');
        }
    }

    if (\is_readable($sAddonPath.'init.php')) {require ($sAddonPath.'init.php');}
    // check if upgrade startet by upgrade-script to echo a message
    $globalStarted = \preg_match('/upgrade\-script\.php$/', $_SERVER["SCRIPT_NAME"]);
    $sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : WB_VERSION);
    Translate::getInstance()->enableAddon(ADMIN_DIRECTORY.'\\addons');
    if (\is_writable(WB_PATH.'/temp/cache')) {
        Translate::getInstance()->clearCache();
    }
    $aRequired['PHP'] = [
          'Addon'  => 'PHP',
          'version' => \PHP_VERSION,
          'required' => '7.4',
          'operator' => '>=',
          ];
    $sTmp   = PreCheck::getMissingRequirements($aRequired);
    if (!empty($sTmp)){$aMsg[] = $sTmp."\n";}
    if (!empty($aMsg)){
//        throw new \InvalidArgumentException(($aMsg));
    }
    unset($aRequired);
    $aRequired['PHP'] = [
          'Addon'  => 'WebsiteBaker',
          'version' => $sWbVersion,
          'required' => '2.13.0',
          'operator' => '>=',
          ];
    $sTmp   = PreCheck::getMissingRequirements($aRequired);
    if (!empty($sTmp)){$aMsg[] = $sTmp."\n";}
    if (!empty($aMsg)){
        $sMsg = ((\is_array($aMsg)) ? (\implode("\n",$aMsg)) : "unknown Error");
        throw new \InvalidArgumentException($sMsg);
    }
    unset($aRequired);


    $OK   = '<span class="ok">OK</span>';
    $FAIL = '<span class="error">FAILED</span>';
    $iErr = false;
    $sNameValPairs    = '';
    $bOldStructure    = false;
//    $sAddonName       = \basename(__DIR__);
    $sTablePrefix      = $database->getTablePrefix();
    $sTableName       = $sTablePrefix.'mod_pagecode_settings';
    $i = (!isset($i) ? 1 : $i);
    $sInstallStruct   = $sAddonPath.'/install-struct.sql.php';
    $sInstallUpdate   = $sAddonPath.'/install-upgrade.sql.php';

//    $dbName           = DB_NAME;
    $iErr = false;
    $aSettings        = [];
    $aNewSettings     = [];
    $aOldSettings     = [];
    $aDefaultSettings = [];

    if (!\is_readable($sInstallUpdate)) {
        $aMsg[] = sprintf("<strong>missing or not readable file [install-upgrade.sql.php]</strong> %s\n",$FAIL);
        $iErr = true;
    } else {
            $selectSettings = (function() use ($database, $sTableName) {
                $aSettings =  [];
                $sql = 'SELECT * FROM `'.$sTableName.'`';
                if ($oRes = $database->query($sql)){
                    $aSettings = $oRes->fetchAll(MYSQLI_ASSOC);
                }
                return ((\count($aSettings) > 0) ? $aSettings : false);
            });

        // prepare an array to create the sql statement
        $prepareSettings = (function($aOldSettings) use ($aSettings)
        {
            $i=0;
            $y=1;
            $iSectionId = 0;
//            $aInvisibile = \explode(',',$aDefaults['invisible']);
            $iSectionId = $aOldSettings[$i]['section_id'];
            $aTmpSettings = [];
            $aTmpSettings[$i]['section_id'] = $iSectionId;
            foreach ($aOldSettings as $aSettings) {
                $sIndexName =  $aSettings['s_name'];
                $sValue     =  $aSettings['s_value'];
                $bChangeSection = ($iSectionId !== $aSettings['section_id']);
                if ($bChangeSection){
                    $i++;
                    $y = 1;
                    $iSectionId =  $aSettings['section_id'];
                    $aTmpSettings[$i]['section_id'] = $iSectionId;
                    $aTmpSettings[$i][$sIndexName] = $sValue;
                } else {
                    $aTmpSettings[$i][$sIndexName] = $sValue;
                }
                $y++;
            }
            return ((\count($aTmpSettings) > 0) ? $aTmpSettings : false);
        });

        // try to create table if not exists
        $database->SqlImport($sInstallStruct, TABLE_PREFIX, true );
        // read settings first
        $sql = 'SELECT * FROM `'.$sTableName.'`';
        // check if table already upgraded
        if ( $bOldStructure = $database->field_exists($sTableName, 'page_id') )
        {
            if (!($aOldSettings = $selectSettings()))
            {
                $aMsg[] = sprintf("[%03d]<strong>Pagescode-Check backup old settings</strong> %s",__LINE__,$FAIL );
                $iErr = true;
            }
        } elseif ($database->field_exists($sTableName, 's_name'))
        {
            // overwrite standardsettings ($aOldSettings)
            if (!($aOldSettings = $selectSettings()))
            {
                $aMsg[] = sprintf("[%03d] <strong>Pagescode-Check has no old settings</strong> %s",__LINE__,$FAIL );
                $iErr = true;
            } else {
                // prepare an array to create the sql statement
                $aOldSettings = $prepareSettings($aOldSettings);
            }
        } // end handling new structure

        if ($database->index_exists($sTableName,'ident_pagecode')){$database->index_remove($sTableName,'ident_pagecode');}
       // delete none allowed entries in settings.s_name
        $aSettingsDenied = ['s_name','s_value']; //'section_id',
        // drop old table and create new one  ident_accord
        if ((\count($aOldSettings)
            && !$iErr && \count($aDefaults))
            && $database->SqlImport($sInstallUpdate, $sTablePrefix, false) //$sInstallUpdate
            ) {
           // restore old settings if there any
            if ($aOldSettings && !$iErr)
            {
                $iSectionId = 0;
                foreach ($aOldSettings as $aItems) {
                    // add new default settings
                    $aSettings  = \array_merge( $aDefaults, $aItems );//replace_recursive

                    $iSectionId = $aSettings['section_id'];
                    // update sections modulename if changed with same table structure
                    $sqlSection = 'UPDATE `'.$sTablePrefix.'sections` SET '
                                . '`module` = \''.$sAddonName.'\' '
                                . 'WHERE `section_id` = '.$iSectionId.';';
                    if (!$database->query($sqlSection)){
                          $aMsg[] = sprintf("[%03d] <strong>Database error table sections  %s</strong>",__LINE__,$database->get_error() );
                          $aMsg[] = sprintf("[%03d] <strong>Pagescode-Check couldn\'t change sections entry ID %s</strong> %s",__LINE__,$iSectionId, $FAIL );

                        }

                    $sNameValPairs = '';
// insert settings per section
                    foreach ($aSettings as $index => $val) {
                        if (\in_array($index, $aSettingsDenied) ) { continue; }

                        $sNameValPairs .= ','."\n".' ('.$iSectionId.', \''.$index.'\', \''.$database->escapeString($val).'\')';
//echo nl2br(sprintf("<div class='w3-white w3-border w3-padding'>[%03d] %s</div>\n",__LINE__, $sNameValPairs));
                    }
                    $sValues = \ltrim($sNameValPairs, ', ');
                    $sTableName = $sTablePrefix.'mod_pagecode_settings';
                    $sql = 'INSERT INTO `'.$sTableName.'` (`section_id`, `s_name`, `s_value`) '
                         . 'VALUES '.$sValues.';';
                    if (!$database->query($sql)) {
                          $aMsg[] = sprintf("[%03d] <strong>Database error table settings %s</strong>",__LINE__,$database->get_error() );
                          $aMsg[] = sprintf("[%03d] <strong>Pagescode-Check  INSERT INTO (restore) old settings</strong> %s",__LINE__,$FAIL );
                          $iErr = true;
                    }
                }  // end for settings per section

            }
            if ($database->is_error()){
                $aMsg[] = sprintf("[%03d] <strong>Database error table settings  %s</strong> %s",__LINE__,$database->get_error(),'' );
                $aMsg[] = sprintf("[%03d] <strong>>Pagescode-Check recreate settings table</strong> %s",__LINE__,$FAIL );
                $iErr = true;
            }
        } else {
            $aMsg[] = sprintf("[%03d] <strong>Database error table settings  %s</strong> %s",__LINE__,$database->get_error(),'' );
            $aMsg[] = sprintf("[%03d] <strong>>Pagescode-Check sqlImport</strong> %s",__LINE__,$FAIL );
            $iErr = true;
        }
        if (!$iErr) {
            $aMsg[] = sprintf("[%03d] <strong>>Pagescode-Check successful updated</strong> %s",__LINE__,$OK );
        } else {
            $aMsg[] = sprintf("[%03d] <strong>>Pagescode-Check couldn't updated</strong> %s",__LINE__,$FAIL );
        }
//        unset($getSettings);
    }
    $aRemoveList = [
        'themes/default/ChildOverview.twig',
        'themes/default/SingleLanguage2.twig'
    ];
   if (!empty($aRemoveList)){
      PreCheck::deleteFiles($sAddonPath,$sAddonPath,$aRemoveList);
    }
  


    if (!$globalStarted && sizeof($aMsg)) {print implode("\n", $aMsg)."\n";}

} catch (\Throwable $ex) {
        /* place to insert different error/logfile messages */
        $aMsg = (\sprintf('[%d] %s', $ex->getLine(), $ex->getMessage()));
    }
//      $sMsg = \sprintf(\implode("\n",$aMsg))."\n";
    $sMsg = ((\is_array($aMsg) && count($aMsg)) ? \sprintf(\implode("\n",$aMsg)) : "")."\n";
    if (\is_array($aMsg) && !empty($aMsg) && $iErr){
        echo nl2br(sprintf("<div class='w3-white w3-border w3-padding'>[%03d] %s</div>\n",__LINE__, $sMsg));
    }
/* */
    if (!$iErr){unset($sMsg);}
    if ($globalStarted) {
//        print Message for upgrade-script
    }
//echo \nl2br(\sprintf("---- [%04d] end of %s \n",__LINE__,$CommandFile));
// end of file
