<?php
/**
 * @category     Addons
 * @package      Admin tool: Pagescode-Check
 * @copyright    Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Org. e.V.
 * @author       Uwe Jacobsen <webdesign@jacobi22.com>, WebsiteBaker Project
 * @license      http://www.gnu.org/licenses/gpl.html   GPL License
 * @version      1.0.2
 * @lastmodified 2024/10/06
 * @since        File available since 2024/10/06
 * @description  info for wbpagecode
 *
 * init.php
 */
/* -------------------------------------------------------- */
// Must include code to stop this file being accessed directly
if (!\defined('SYSTEM_RUN')) {\header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found'); echo '404 Not Found'; \flush(); exit;}
/* -------------------------------------------------------- */
// only included in init.php may be needed as global if script is called within a function
global  $sAppPath,
//        $sAddonPath,
        $sAddonUrl,
        $sAddonName,
        $sAddonThemeUrl,
        $sAddonTemplateUrl,
        $sAddonThemePath,
        $sAddonRel,
        $aExtraFields,
        $oReg;
/* -------------------------------------------------------- */
// folderstructure steps to modules path
$sAppPath = dirname(dirname(__DIR__));
// load config created for modify or save module files if WB_PATH don't exist, '
if (!defined('WB_PATH')) {
    require($sAppPath . '/config.php');
}
// Must include code to prevent this file from being accessed directly
if (defined('WB_PATH') == false) {
    die('Cannot access ' . basename(__DIR__) . '/' . basename(__FILE__) . ' directly');
}
/* -------------------------------------------------------- */

$sCallingScript = $oReg->Request->getServerVar('SCRIPT_NAME');
$globalStarted = \preg_match('/upgrade\-script\.php$/', $sCallingScript);
$sWbVersion = ($globalStarted && \defined('VERSION') ? VERSION : WB_VERSION);

// needed for simple Dispatcher
$sAddonName = basename($sAddonPath);
$bIsBackend = ($oApp instanceof admin);

// An associative array that by default contains the contents of $_GET, $_POST and $_COOKIE.
$aRequestVars = $_REQUEST;

if (!function_exists('index_exists')) {

    function index_exists($table_name, $index_name, $number_fields = 0) {
        global $database;
        $number_fields = intval($number_fields);
        $keys = 0;
        $sql = 'SHOW INDEX FROM `' . $table_name . '`';
        if (($res_keys = $database->query($sql))) {
            while (($rec_key = $res_keys->fetchRow(MYSQLI_ASSOC))) {
                if (($rec_key['Key_name'] || ($rec_key['Column_name']) == $index_name)) {
                    $keys++;
                }
            }
        }
        if ($number_fields == 0) {
            return ($keys != $number_fields);
        } else {
            return ($keys == $number_fields);
        }
    }

}

if (!function_exists('UpdateKeyValue')) {

    function UpdateKeyValue($table, $key, $value = '', array $aExtraFields = []) {
        global $database;
        if (!is_array($key)) {
            if (\trim($key) != '') {
                $key = array(\trim($key) => \trim($value));
            } else {
                $key = [];
            }
        }
        $retval = [];
        $iFirst = 0;
        $aIndexField = [];
        $sExtraFields = '';
        $aTmp = [];
        $sql = 'SHOW FIELDS FROM `' . TABLE_PREFIX . $table . '` ';
        if ($oRes = $database->query($sql)) {
            while ($aRes = $oRes->fetchRow(MYSQLI_ASSOC)) {
                if (!isset($aExtraFields[$aRes['Field']])) {
                    continue;
                }
                $aTmp[$aRes['Field']] = $aExtraFields[$aRes['Field']];
                if (index_exists(TABLE_PREFIX . $table, $aRes['Field'])) {
                    $aIndexField[] = $aRes['Field'] . ' = ' . $aTmp[$aRes['Field']];
                    $iFirst++;
                }
            }
            $aExtraFieldsList = \array_intersect_key($aTmp, $aExtraFields);
            foreach ($aExtraFieldsList as $sName => $sValue) {
                $sExtraFields .= '`' . $sName . '` = \'' . $sValue . '\', ';
            }
        } else {
            $retval[] = $sql;
            $retval[] = $database->get_error();
        }
        foreach ($key as $index => $val) {
//            $index = strtolower($index);
            $sql = 'SELECT COUNT(*) FROM `' . TABLE_PREFIX . $table . '` '
                    . 'WHERE `s_name` = \'' . $index . '\' '
                    . (($aIndexField['0'] != '') ? 'AND ' . $aIndexField['0'] : '');

            if ($database->get_one($sql)) {
                $sql = 'UPDATE ';
                $sql_where = 'WHERE `s_name` LIKE \'' . $index . '\'';
            } else {
                $sql = 'INSERT INTO ';
                $sql_where = '';
            }
            $sql .= '`' . TABLE_PREFIX . $table . '` SET ';
            $sql .= (($sExtraFields) ? $sExtraFields : '');
            $sql .= '`s_name` = \'' . $index . '\', ';
            $sql .= '`s_value` = \'' . $val . '\' ' . $sql_where;
            if (!$database->query($sql)) {
                $retval[] = $sql;
                $retval[] = $database->get_error();
            }
        }
        return ((\sizeof($retval) == 0) ? true : $retval);
    }

;
}
#################################################################################

$sModule = '
<script type="text/javascript">
var ' . strtoupper($sAddonName) . ' = {
        ADMIN_DIRECTORY : "' . ADMIN_DIRECTORY . '",
        WB_URL : "' . $oReg->AppUrl . '",
        ADMIN_URL : "' . $oReg->AcpUrl . '",
        AddonUrl : "' . $sAddonUrl . '/",
        MODULE_THEME_URL : "' . $sAddonThemeUrl . '"
    };
    var ModulesCss = "' . $sAddonThemeUrl . '/css/default.css";
    if (typeof LazyOnLoad === \'undefined\'){
        $.insert(ModulesCss);
    } else {
        LazyOnLoad (\'head\', ModulesCss);
    }


</script>
';

$aDefaults = [
    'section_id' => '0',
    'page_id' => '0'
];
// set any defaults
$sAddonName = basename($sAddonPath);
$sAddonPath = $oReg->AppPath . 'modules/' . $sAddonName;
$sAddonUrl = $oReg->AppUrl . 'modules/' . $sAddonName;
$sAddonRel = '/modules/' . $sAddonName;
$aInstance = [
    'ADDON_NAME' => strtoupper($sAddonName),
    'ADMIN_DIRECTORY' => ADMIN_DIRECTORY,
    'AppUrl' => $oReg->AppUrl,
    'AcpUrl' => $oReg->AcpUrl,
    'sAddonUrl' => $sAddonUrl,
    'sAddonThemeUrl' => $sAddonUrl . '/themes/default',
    'ModulesTemplateUrl' => ($bIsBackend ? $sAddonThemeUrl : $sAddonTemplateUrl),
    'ADDON_URL' => $sAddonUrl,
    'ModuleScript' => $sModule
];

// end of file

